Package: geex
Type: Package
Title: An API for M-Estimation
Version: 1.1.1
Authors@R: c(person("Bradley", "Saul", role = c("aut", "cre"),
                    email = "bradleysaul@gmail.com"),
             person("Brian", "Barkley", role = c("ctb")))
Author: Bradley Saul [aut, cre],
  Brian Barkley [ctb]
Maintainer: Bradley Saul <bradleysaul@gmail.com>
Description: Provides a general, flexible framework for estimating parameters
    and empirical sandwich variance estimator from a set of unbiased estimating
    equations (i.e., M-estimation in the vein of Stefanski & Boos (2002)
    <doi:10.1198/000313002753631330>). All examples from Stefanski & Boos (2002)
    are published in the corresponding Journal of Statistical Software paper
    "The Calculus of M-Estimation in R with geex" by Saul & Hudgens (2020)
    <doi:10.18637/jss.v092.i02>. Also provides an API to compute finite-sample 
    variance corrections.
Depends: R (>= 3.3)
Imports: Matrix (>= 1.2-6), rootSolve (>= 1.6.6), numDeriv (>=
        2014.2-1), lme4 (>= 1.1-12), methods (>= 3.3)
Suggests: testthat, knitr, dplyr, moments, sandwich, inferference,
        xtable, AER, ICSNP, MASS, gee, saws, rmarkdown, geepack, covr,
        mvtnorm
URL: https://github.com/bsaul/geex, https://bsaul.github.io/geex/
BugReports: https://github.com/bsaul/geex/issues
License: MIT + file LICENSE
LazyData: TRUE
VignetteBuilder: knitr
RoxygenNote: 7.2.1
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2022-07-24 16:01:38 UTC; bradley.saul
Repository: CRAN
Date/Publication: 2022-08-08 10:40:04 UTC
Built: R 4.6.0; ; 2026-04-26 06:52:37 UTC; unix
