# Generated by roxygen2: do not edit by hand

S3method(grab_psiFUN,geeglm)
S3method(grab_psiFUN,glm)
S3method(grab_psiFUN,merMod)
export(compute_sigma)
export(correct_by)
export(correction)
export(create_GFUN)
export(create_basis)
export(create_psiFUN_list)
export(diagnose_roots)
export(estimate_GFUN_roots)
export(estimate_sandwich_matrices)
export(fay_bias_correction)
export(fay_df_correction)
export(get_corrections)
export(grab)
export(grab_GFUN)
export(grab_bread)
export(grab_bread_list)
export(grab_design_levels)
export(grab_design_matrix)
export(grab_ee_list)
export(grab_estFUN)
export(grab_fixed_formula)
export(grab_meat)
export(grab_meat_list)
export(grab_psiFUN)
export(grab_psiFUN_list)
export(grab_response)
export(grab_response_formula)
export(m_estimate)
export(roots)
export(setup_approx_control)
export(setup_control)
export(setup_deriv_control)
export(setup_root_control)
export(show)
exportClasses(approx_control)
exportClasses(basic_control)
exportClasses(correct_control)
exportClasses(deriv_control)
exportClasses(estimating_function)
exportClasses(geex)
exportClasses(geex_control)
exportClasses(geex_summary)
exportClasses(m_estimation_basis)
exportClasses(root_control)
exportClasses(sandwich_components)
exportMethods(coef)
exportMethods(get_corrections)
exportMethods(nobs)
exportMethods(show)
exportMethods(summary)
exportMethods(vcov)
exportMethods(weights)
importFrom(methods,formalArgs)
